@@ -7,7 +7,7 @@
 --
 -- C/C++ support for Lua extensions is provided.
 --
--- Available from LuaRocks as 'ldoc' and as a [Zip file](http://stevedonovan.github.com/files/ldoc-1.3.9.zip)
+-- Available from LuaRocks as 'ldoc' and as a [Zip file](http://stevedonovan.github.com/files/ldoc-1.4.2.zip)
 --
 -- [Github Page](https://github.com/stevedonovan/ldoc)
 --
@@ -25,6 +25,8 @@ local List = require 'pl.List'
 local stringx = require 'pl.stringx'
 local tablex = require 'pl.tablex'
 
+-- Penlight compatibility
+utils.unpack = utils.unpack or unpack or table.unpack
 
 local append = table.insert
 
@@ -35,7 +37,7 @@ app.require_here()
 
 --- @usage
 local usage = [[
-ldoc, a documentation generator for Lua, vs 1.4.0
+ldoc, a documentation generator for Lua, vs 1.4.2
   -d,--dir (default doc) output directory
   -o,--output  (default 'index') output name
   -v,--verbose          verbose
@@ -50,6 +52,7 @@ ldoc, a documentation generator for Lua, vs 1.4.0
   -b,--package  (default .) top-level package basename (needed for module(...))
   -x,--ext (default html) output file extension
   -c,--config (default config.ld) configuration name
+  -u,--unqualified     don't show package name in sidebar links
   -i,--ignore ignore any 'no doc comment or no module' warnings
   -X,--not_luadoc break LuaDoc compatibility. Descriptions may continue after tags.
   -D,--define (default none) set a flag to be used in config.ld
@@ -80,22 +83,15 @@ local Item,File,Module = doc.Item,doc.File,doc.Module
 local quit = utils.quit
 
 
-class.ModuleMap(KindMap)
-local ModuleMap = ModuleMap
+local ModuleMap = class(KindMap)
+doc.ModuleMap = ModuleMap
 
 function ModuleMap:_init ()
    self.klass = ModuleMap
    self.fieldname = 'section'
 end
 
-ModuleMap:add_kind('function','Functions','Parameters')
-ModuleMap:add_kind('table','Tables','Fields')
-ModuleMap:add_kind('field','Fields')
-ModuleMap:add_kind('lfunction','Local Functions','Parameters')
-ModuleMap:add_kind('annotation','Issues')
-
-
-class.ProjectMap(KindMap)
+local ProjectMap = class(KindMap)
 ProjectMap.project_level = true
 
 function ProjectMap:_init ()
@@ -103,11 +99,7 @@ function ProjectMap:_init ()
    self.fieldname = 'type'
 end
 
-ProjectMap:add_kind('module','Modules')
-ProjectMap:add_kind('script','Scripts')
-ProjectMap:add_kind('classmod','Classes')
-ProjectMap:add_kind('topic','Topics')
-ProjectMap:add_kind('example','Examples')
+
 
 local lua, cc = lang.lua, lang.cc
 
@@ -122,15 +114,58 @@ local file_types = {
    ['.mm'] = cc,
    ['.moon'] = lang.moon,
 }
-
 ------- ldoc external API ------------
 
 -- the ldoc table represents the API available in `config.ld`.
 local ldoc = { charset = 'UTF-8' }
-local add_language_extension
 
-local function override (field)
-   if ldoc[field] ~= nil then args[field] = ldoc[field] end
+local known_types, kind_names = {}
+
+local function lookup (itype,igroup,isubgroup)
+   local kn = kind_names[itype]
+   known_types[itype] = true
+   if kn then
+      if type(kn) == 'string' then
+         igroup = kn
+      else
+         igroup = kn[1]
+         isubgroup = kn[2]
+      end
+   end
+   return itype, igroup, isubgroup
+end
+
+local function setup_kinds ()
+   kind_names = ldoc.kind_names or {}
+
+   ModuleMap:add_kind(lookup('function','Functions','Parameters'))
+   ModuleMap:add_kind(lookup('table','Tables','Fields'))
+   ModuleMap:add_kind(lookup('field','Fields'))
+   ModuleMap:add_kind(lookup('lfunction','Local Functions','Parameters'))
+   ModuleMap:add_kind(lookup('annotation','Issues'))
+
+   ProjectMap:add_kind(lookup('module','Modules'))
+   ProjectMap:add_kind(lookup('script','Scripts'))
+   ProjectMap:add_kind(lookup('classmod','Classes'))
+   ProjectMap:add_kind(lookup('topic','Topics'))
+   ProjectMap:add_kind(lookup('example','Examples'))
+
+   for k in pairs(kind_names) do
+      if not known_types[k] then
+         quit("unknown item type "..tools.quote(k).." in kind_names")
+      end
+   end
+end
+
+
+local add_language_extension
+-- hacky way for doc module to be passed options...
+doc.ldoc = ldoc
+
+-- if the corresponding argument was the default, then any ldoc field overrides
+local function override (field,defval)
+   defval = defval or false
+   if args[field] == defval and ldoc[field] ~= nil then args[field] = ldoc[field] end
 end
 
 -- aliases to existing tags can be defined. E.g. just 'p' for 'param'
@@ -149,6 +184,8 @@ function ldoc.tparam_alias (name,type)
    ldoc.alias(name,{'param',modifiers={type=type}})
 end
 
+ldoc.alias ('error',doc.error_macro)
+
 ldoc.tparam_alias 'string'
 ldoc.tparam_alias 'number'
 ldoc.tparam_alias 'int'
@@ -186,13 +223,14 @@ function ldoc.custom_see_handler(pat, handler)
 end
 
 local ldoc_contents = {
-   'alias','add_language_extension','new_type','add_section', 'tparam_alias',
+   'alias','add_language_extension','custom_tags','new_type','add_section', 'tparam_alias',
    'file','project','title','package','format','output','dir','ext', 'topics',
    'one','style','template','description','examples', 'pretty', 'charset', 'plain',
-   'readme','all','manual_url', 'ignore', 'colon', 'sort', 'module_file',
-   'boilerplate','merge', 'wrap', 'not_luadoc', 'template_escape',
+   'readme','all','manual_url', 'ignore', 'colon', 'sort', 'module_file','vars',
+   'boilerplate','merge', 'wrap', 'not_luadoc', 'template_escape','merge_error_groups',
    'no_return_or_parms','no_summary','full_description','backtick_references', 'custom_see_handler',
-   'no_space_before_args',
+   'no_space_before_args','parse_extra','no_lua_ref','sort_modules','use_markdown_titles',
+   'unqualified', 'custom_display_name_handler', 'kind_names', 'custom_references',
 }
 ldoc_contents = tablex.makeset(ldoc_contents)
 
@@ -299,6 +337,16 @@ else
    args.file = abspath(args.file)
 end
 
+if type(ldoc.custom_tags) == 'table' then -- custom tags
+  for i, custom in ipairs(ldoc.custom_tags) do
+    if type(custom) == 'string' then
+      custom = {custom}
+      ldoc.custom_tags[i] = custom
+    end
+    doc.add_tag(custom[1], 'ML')
+  end
+end -- custom tags
+
 local source_dir = args.file
 if type(source_dir) == 'table' then
    source_dir = source_dir[1]
@@ -318,6 +366,8 @@ source_dir = source_dir:gsub('[/\\]%.$','')
 --  * 'NAME' explicitly give the base module package name
 --
 
+override ('package','.')
+
 local function setup_package_base()
    if ldoc.package then args.package = ldoc.package end
    if args.package == '.' then
@@ -349,6 +399,7 @@ local function process_file (f, flist)
    local ftype = file_types[ext]
    if ftype then
       if args.verbose then print(f) end
+      ftype.extra = ldoc.parse_extra or {}
       local F,err = parse.file(f,ftype,args)
       if err then
          if F then
@@ -368,6 +419,18 @@ override 'colon'
 override 'merge'
 override 'not_luadoc'
 override 'module_file'
+override 'boilerplate'
+
+setup_kinds()
+
+-- LDoc is doing plain ole C, don't want random Lua references!
+if ldoc.parse_extra and ldoc.parse_extra.C then
+   ldoc.no_lua_ref = true
+end
+
+if ldoc.merge_error_groups == nil then
+   ldoc.merge_error_groups = 'Error Message'
+end
 
 -- ldoc.module_file establishes a partial ordering where the
 -- master module files are processed first.
@@ -429,9 +492,10 @@ else
    quit ("file or directory does not exist: "..quote(args.file))
 end
 
+
 -- create the function that renders text (descriptions and summaries)
 -- (this also will initialize the code prettifier used)
-override 'format'
+override ('format','plain')
 override 'pretty'
 ldoc.markup = markup.create(ldoc, args.format,args.pretty)
 
@@ -469,7 +533,8 @@ if type(ldoc.examples) == 'table' then
       })
       -- wrap prettify for this example so it knows which file to blame
       -- if there's a problem
-      item.postprocess = function(code) return prettify.lua(f,code,0,true) end
+      local ext = path.extension(f):sub(2)
+      item.postprocess = function(code) return prettify.lua(ext,f,code,0,true) end
    end)
 end
 
@@ -491,6 +556,9 @@ if type(ldoc.readme) == 'table' then
       -- headers in the readme, which are attached to the File. So
       -- we pass the File to the postprocesser, which will insert the section markers
       -- and resolve inline @ references.
+      if ldoc.use_markdown_titles then
+         item.display_name = F.display_name
+      end
       item.postprocess = function(txt) return ldoc.markup(txt,F) end
    end)
 end
@@ -520,19 +588,17 @@ for mod in module_list:iter() do
    project:add(mod,module_list)
 end
 
--- the default is not to show local functions in the documentation.
-if not args.all and not ldoc.all then
-   for mod in module_list:iter() do
-      mod:mask_locals()
-   end
-end
+override 'all'
 
-table.sort(module_list,function(m1,m2)
-   return m1.name < m2.name
-end)
+if ldoc.sort_modules then
+   table.sort(module_list,function(m1,m2)
+      return m1.name < m2.name
+   end)
+end
 
 ldoc.single = modcount == 1 and first_module or nil
 
+--do return end
 
 -------- three ways to dump the object graph after processing -----
 
@@ -543,8 +609,12 @@ if args.module then
    if args.module == true then
       file_list[1]:dump(args.verbose)
    else
-      local fun = module_list[1].items.by_name[args.module]
-      if not fun then quit(quote(args.module).." is not part of "..quote(args.file)) end
+      local M,name = module_list[1], args.module
+      local fun = M.items.by_name[name]
+      if not fun then
+         fun = M.items.by_name[M.mod_name..':'..name]
+      end
+      if not fun then quit(quote(name).." is not part of "..quote(args.file)) end
       fun:dump(true)
    end
    return
@@ -575,15 +645,35 @@ if args.filter ~= 'none' then
    os.exit()
 end
 
+-- can specify format, output, dir and ext in config.ld
+override ('output','index')
+override ('dir','doc')
+override ('ext','html')
+override 'one'
+
+-- handling styling and templates --
 ldoc.css, ldoc.templ = 'ldoc.css','ldoc.ltp'
 
+-- special case: user wants to generate a .md file from a .lua file
 if args.ext == 'md' then
-   ldoc.templ = 'ldoc.mdtp'
+   if #module_list ~= 1 then
+      quit("can currently only generate Markdown output from one module only")
+   end
+   if ldoc.template == '!' then
+      ldoc.template = '!md'
+   end
+   args.output = module_list[1].name
+   args.dir = '.'
    ldoc.template_escape = '>'
    ldoc.style = false
    args.ext = '.md'
 end
 
+local function match_bang (s)
+   if type(s) ~= 'string' then return end
+   return s:match '^!(.*)'
+end
+
 local function style_dir (sname)
    local style = ldoc[sname]
    local dir
@@ -594,7 +684,7 @@ local function style_dir (sname)
    if style then
       if style == true then
          dir = config_dir
-      elseif type(style) == 'string' and path.isdir(style) then
+      elseif type(style) == 'string' and (path.isdir(style) or match_bang(style)) then
          dir = style
       else
          quit(quote(tostring(style)).." is not a directory")
@@ -603,7 +693,6 @@ local function style_dir (sname)
    end
 end
 
-
 -- the directories for template and stylesheet can be specified
 -- either by command-line '--template','--style' arguments or by 'template and
 -- 'style' fields in config.ld.
@@ -614,35 +703,41 @@ end
 style_dir 'style'
 style_dir 'template'
 
--- can specify format, output, dir and ext in config.ld
-override 'output'
-override 'dir'
-override 'ext'
-override 'one'
-override 'boilerplate'
-
 if not args.ext:find '^%.' then
    args.ext = '.'..args.ext
 end
 
 if args.one then
-   ldoc.css = 'ldoc_one.css'
+   ldoc.style = '!one'
 end
 
-if args.style == '!' or args.template == '!' then
+local builtin_style, builtin_template = match_bang(args.style),match_bang(args.template)
+if builtin_style or builtin_template then
    -- '!' here means 'use built-in templates'
    local tmpdir = path.join(path.is_windows and os.getenv('TMP') or '/tmp','ldoc')
    if not path.isdir(tmpdir) then
       lfs.mkdir(tmpdir)
    end
    local function tmpwrite (name)
-      utils.writefile(path.join(tmpdir,name),require('ldoc.html.'..name:gsub('%.','_')))
+      local ok,text = pcall(require,'ldoc.html.'..name:gsub('%.','_'))
+      if not ok then
+         quit("cannot find builtin template "..name..": "..text)
+      end
+      if not utils.writefile(path.join(tmpdir,name),text) then
+         quit("cannot write to temp directory "..tmpdir)
+      end
    end
-   if args.style == '!' then
+   if builtin_style then
+      if builtin_style ~= '' then
+         ldoc.css = 'ldoc_'..builtin_style..'.css'
+      end
       tmpwrite(ldoc.css)
       args.style = tmpdir
    end
-   if args.template == '!' then
+   if builtin_template then
+      if builtin_template ~= '' then
+         ldoc.templ = 'ldoc_'..builtin_template..'.ltp'
+      end
       tmpwrite(ldoc.templ)
       args.template = tmpdir
    end
