@@ -21,7 +21,7 @@ local TAG_MULTI,TAG_ID,TAG_SINGLE,TAG_TYPE,TAG_FLAG,TAG_MULTI_LINE = 'M','id','S
 --  - 'N' tags which have no associated value, like 'local` (TAG_FLAG)
 --  - 'T' tags which represent a type, like 'function' (TAG_TYPE)
 local known_tags = {
-   param = 'M', see = 'M', usage = 'ML', ['return'] = 'M', field = 'M', author='M',set='M';
+   param = 'M', see = 'M', comment = 'M', usage = 'ML', ['return'] = 'M', field = 'M', author='M',set='M';
    class = 'id', name = 'id', pragma = 'id', alias = 'id', within = 'id',
    copyright = 'S', summary = 'S', description = 'S', release = 'S', license = 'S',
    fixme = 'S', todo = 'S', warning = 'S', raise = 'S', charset = 'S',
@@ -121,7 +121,6 @@ function doc.module_info_tags ()
    return List.iter(known_tags._module_info)
 end
 
-
 -- annotation tags can appear anywhere in the code and may contain any of these tags:
 known_tags._annotation_tags = {
    fixme = true, todo = true, warning = true
@@ -130,17 +129,20 @@ known_tags._annotation_tags = {
 local acount = 1
 
 function doc.expand_annotation_item (tags, last_item)
-   if tags.summary ~= '' then return false end
+   if tags.summary ~= '' or last_item == nil then return false end
+   local item_name = last_item.tags.name
    for tag, value in pairs(tags) do
       if known_tags._annotation_tags[tag] then
          tags:add('class','annotation')
          tags:add('summary',value)
-         local item_name = last_item and last_item.tags.name or '?'
          tags:add('name',item_name..'-'..tag..acount)
          acount = acount + 1
          return true
+      elseif tag == 'return' then
+         last_item:set_tag(tag,value)
       end
    end
+   return false
 end
 
 -- we process each file, resulting in a File object, which has a list of Item objects.
@@ -174,6 +176,7 @@ function File:export_item (name)
    for item in self.items:iter() do
       local tags = item.tags
       if tags.name == name then
+         tags.export = true
          if tags['local'] then
             tags['local'] = nil
          end
@@ -205,17 +208,18 @@ function File:find_module_in_files (name)
    end
 end
 
+local function init_within_section (mod,name)
+   mod.kinds:add_kind(name, name)
+   mod.enclosing_section = mod.section
+   mod.section = nil
+   return name
+end
+
 function File:finish()
    local this_mod
    local items = self.items
    local tagged_inside
-   local function add_section (item, display_name)
-      display_name = display_name or item.display_name
-      this_mod.section = item
-      this_mod.kinds:add_kind(display_name,display_name..' ',nil,item)
-      this_mod.sections:append(item)
-      this_mod.sections.by_name[display_name:gsub('%A','_')] = item
-   end
+   self.args = self.args or {}
    for item in items:iter() do
       if mod_section_type(this_mod) == 'factory' and item.tags then
          local klass = '@{'..this_mod.section.name..'}'
@@ -227,7 +231,10 @@ function File:finish()
          end
       end
       item:finish()
-      if doc.project_level(item.type) then
+      -- the default is not to show local functions in the documentation.
+      if not self.args.all and (item.type=='lfunction' or (item.tags and item.tags['local'])) then
+         -- don't add to the module --
+      elseif doc.project_level(item.type) then
          this_mod = item
          local package,mname,submodule
          if item.type == 'module' then
@@ -262,7 +269,7 @@ function File:finish()
          if not submodule then
             this_mod.package = package
             this_mod.mod_name = mname
-            this_mod.kinds = ModuleMap() -- the iterator over the module contents
+            this_mod.kinds = doc.ModuleMap() -- the iterator over the module contents
             self.modules:append(this_mod)
          end
       elseif doc.section_tag(item.type) then
@@ -271,21 +278,30 @@ function File:finish()
             this_mod.section = nil
          else
             local summary = item.summary:gsub('%.$','')
+            local lookup_name
             if doc.class_tag(item.type) then
                display_name = 'Class '..item.name
+               lookup_name = item.name
                item.module = this_mod
                this_mod.items.by_name[item.name] = item
             else
                display_name = summary
+               lookup_name = summary
             end
             item.display_name = display_name
-            add_section(item)
+            this_mod.section = item
+            this_mod.kinds:add_kind(display_name,display_name..' ',nil,item)
+            this_mod.sections:append(item)
+            this_mod.sections.by_name[lookup_name:gsub('%A','_')] = item
          end
       else
          local to_be_removed
          -- add the item to the module's item list
          if this_mod then
             -- new-style modules will have qualified names like 'mod.foo'
+            if item.name == nil then
+               self:error("item's name is nil")
+            end
             local mod,fname = split_dotted_name(item.name)
             -- warning for inferred unqualified names in new style modules
             -- (retired until we handle methods like Set:unset() properly)
@@ -302,15 +318,11 @@ function File:finish()
                item.name = fname
             end
 
-            local enclosing_section
             if tagged_inside then
                item.tags.within = tagged_inside
             end
             if item.tags.within then
-               local name = item.tags.within
-               this_mod.kinds:add_kind(name, name)
-               enclosing_section = this_mod.section
-               this_mod.section = nil
+               init_within_section(this_mod,item.tags.within)
             end
 
             -- right, this item was within a section or a 'class'
@@ -326,11 +338,24 @@ function File:finish()
                -- if it was a class, then if the name is unqualified then it becomes
                -- 'Class:foo' (unless flagged as being a constructor, static or not a function)
                if doc.class_tag(stype) or classmod then
-                  if not item.name:match '[:%.]' then -- not qualified
+                  if not item.name:match '[:%.]' then -- not qualified name!
+                     -- a class is either a @type section or a @classmod module. Is this a _method_?
                      local class = classmod and this_mod.name or this_section.name
-                     local lang = this_mod.file.lang
                      local static = item.tags.constructor or item.tags.static or item.type ~= 'function'
-                     item.name = class..(not static and lang.method_call or '.')..item.name
+                     -- methods and metamethods go into their own special sections...
+                     if classmod and item.type == 'function' then
+                        local inferred_section
+                        if item.name:match '^__' then
+                           inferred_section = 'Metamethods'
+                        elseif not static then
+                           inferred_section = 'Methods'
+                        end
+                        if inferred_section then
+                           item.tags.within = init_within_section(this_mod,inferred_section)
+                        end
+                     end
+                     -- Whether to use '.' or the language's version of ':' (e.g. \ for Moonscript)
+                     item.name = class..(not static and this_mod.file.lang.method_call or '.')..item.name
                   end
                   if stype == 'factory'  then
                      if item.tags.private then to_be_removed = true
@@ -349,14 +374,16 @@ function File:finish()
                   section_description = item.tags.within
                   item.section = section_description
                else
-                  section_description = "Methods"
+                  if item.type == 'function' or item.type == 'lfunction' then
+                     section_description = "Methods"
+                  end
                   item.section = item.type
                end
-            elseif item.tags.within then
+            elseif item.tags.within then -- ad-hoc section...
                section_description = item.tags.within
                item.section = section_description
             else -- otherwise, just goes into the default sections (Functions,Tables,etc)
-               item.section = item.type
+               item.section = item.type;
             end
 
             item.module = this_mod
@@ -368,7 +395,10 @@ function File:finish()
             end
 
             -- restore current section after a 'within'
-            if enclosing_section then this_mod.section = enclosing_section end
+            if this_mod.enclosing_section then
+               this_mod.section = this_mod.enclosing_section
+               this_mod.enclosing_section = nil
+            end
 
          else
             -- must be a free-standing function (sometimes a problem...)
@@ -405,10 +435,7 @@ function Item:_init(tags,file,line)
    self.tags = {}
    self.formal_args = tags.formal_args
    tags.formal_args = nil
-   local iter = tags.iter
-   if not iter then
-      iter = Map.iter
-   end
+   local iter = tags.iter or Map.iter
    for tag in iter(tags) do
       self:set_tag(tag,tags[tag])
    end
@@ -422,16 +449,31 @@ end
 
 function Item:trailing_warning (kind,tag,rest)
    if type(rest)=='string' and #rest > 0 then
-      Item.warning(self,kind.." tag: '"..tag..'" has trailing text; use not_luadoc=true if you want description to continue between tags\n'..rest)
+      Item.warning(self,kind.." tag: '"..tag..'" has trailing text ; use not_luadoc=true if you want description to continue between tags\n"'..rest..'"')
    end
 end
 
+local function is_list (l)
+   return getmetatable(l) == List
+end
+
 function Item:set_tag (tag,value)
    local ttype = known_tags[tag]
    local args = self.file.args
 
    if ttype == TAG_MULTI or ttype == TAG_MULTI_LINE then -- value is always a List!
-      if getmetatable(value) ~= List then
+      local ovalue = self.tags[tag]
+      if ovalue then -- already defined, must be a list
+         --print(tag,ovalue,value)
+         if is_list(value) then
+            ovalue:extend(value)
+         else
+            ovalue:append(value)
+         end
+         value = ovalue
+      end
+      -- these multiple values are always represented as lists
+      if not is_list(value) then
          value = List{value}
       end
       if ttype ~= TAG_MULTI_LINE and args and args.not_luadoc then
@@ -448,7 +490,11 @@ function Item:set_tag (tag,value)
       if type(value) == 'table' then
          if value.append then -- it was a List!
             -- such tags are _not_ multiple, e.g. name
-            self:error("'"..tag.."' cannot have multiple values")
+            if tag == 'class' and value:contains 'example' then
+               self:error("cannot use 'example' tag for functions or tables. Use 'usage'")
+            else
+               self:error("'"..tag.."' cannot have multiple values; "..tostring(value))
+            end
          end
          value = value[1]
          modifiers = value.modifiers
@@ -482,7 +528,7 @@ function Item.check_tag(tags,tag, value, modifiers)
    if alias then
       if type(alias) == 'string' then
          tag = alias
-      else
+      elseif type(alias) == 'table' then --{ tag, value=, modifiers = }
          local avalue,amod
          tag, avalue, amod = alias[1],alias.value,alias.modifiers
          if avalue then value = avalue..' '..value end
@@ -496,6 +542,8 @@ function Item.check_tag(tags,tag, value, modifiers)
                modifiers[m] = v
             end
          end
+      else -- has to be a function that at least returns tag, value
+         return alias(tags,value,modifiers)
       end
    end
    local ttype = known_tags[tag]
@@ -547,6 +595,13 @@ end
 
 local build_arg_list, split_iden  -- forward declaration
 
+function Item:split_param (line)
+   local name, comment = line:match('%s*([%w_%.:]+)(.*)')
+   if not name then
+      self:error("bad param name format '"..line.."'. Are you missing a parameter name?")
+   end
+   return name, comment
+end
 
 function Item:finish()
    local tags = self.tags
@@ -595,10 +650,7 @@ function Item:finish()
       local param_names, comments = List(), List()
       if params then
          for line in params:iter() do
-            local name, comment = line:match('%s*([%w_%.:]+)(.*)')
-            if not name then
-               self:error("bad param name format '"..line.."'. Are you missing a parameter name?")
-            end
+            local name, comment = self:split_param(line)
             param_names:append(name)
             comments:append(comment)
          end
@@ -614,18 +666,19 @@ function Item:finish()
       if fargs then
          if #param_names == 0 then
             --docs may be embedded in argument comments; in either case, use formal arg names
-            formal = List()
-            if fargs.return_comment then
-               local retc = self:parse_argument_comment(fargs.return_comment,'return')
-               self.ret = List{retc}
-            end
-            for i, name in ipairs(fargs) do
-               formal:append(name)
-               comments:append(self:parse_argument_comment(fargs.comments[name],self.parameter))
-            end
-         elseif #fargs > 0 then
+            local ret
+            formal,comments,ret = self:parse_formal_arguments(fargs)
+            if ret and not self.ret then self.ret = ret end
+         elseif #fargs > 0 then -- consistency check!
             local varargs = fargs[#fargs] == '...'
             if varargs then table.remove(fargs) end
+            if tags.export then
+               if fargs[1] == 'self' then
+                  table.remove(fargs,1)
+               else
+                  tags.static = true
+               end
+            end
             local k = 0
             for _,pname in ipairs(param_names) do
                local _,field = split_iden(pname)
@@ -641,14 +694,27 @@ function Item:finish()
                end
             end
             if k < #fargs then
-               for i = k+1,#fargs do
-                  if fargs[i] ~= '...' then
-                     self:warning("undocumented formal argument: "..quote(fargs[i]))
-                  end
+               for i = k+1,#fargs do if fargs[i] ~= '...' then
+                  self:warning("undocumented formal argument: "..quote(fargs[i]))
+               end end
+            end
+         end -- #fargs > 0
+         -- formal arguments may come with types, inferred by the
+         -- appropriate code in ldoc.lang
+         if fargs.types then
+            self.modifiers[field] = List()
+            for t in fargs.types:iter() do
+               self:add_type(field,t)
+            end
+            if fargs.return_type then
+               if not self.ret then -- type, but no comment; no worries
+                  self.ret = List{''}
                end
+               self.modifiers['return'] = List()
+               self:add_type('return',fargs.return_type)
             end
          end
-      end
+      end -- fargs
 
       -- the comments are associated with each parameter by
       -- adding name-value pairs to the params list (this is
@@ -684,6 +750,13 @@ function Item:finish()
       self.params = params
       self.args = build_arg_list (names,pmods)
    end
+   if self.ret then
+      self:build_return_groups()
+   end
+end
+
+function Item:add_type(field,type)
+   self.modifiers[field]:append {type = type}
 end
 
 -- ldoc allows comments in the formal arg list to be used, if they aren't specified with @param
@@ -694,13 +767,26 @@ function Item:parse_argument_comment (comment,field)
       comment = comment:gsub('^%-+%s*','')
       local type,rest = comment:match '([^:]+):(.*)'
       if type then
-         self.modifiers[field]:append {type = type}
+         self:add_type(field,type)
          comment = rest
       end
    end
    return comment or ''
 end
 
+function Item:parse_formal_arguments (fargs)
+   local formal, comments, ret = List(), List()
+   if fargs.return_comment then
+      local retc = self:parse_argument_comment(fargs.return_comment,'return')
+      ret = List{retc}
+   end
+   for i, name in ipairs(fargs) do
+      formal:append(name)
+      comments:append(self:parse_argument_comment(fargs.comments[name],self.parameter))
+   end
+   return formal, comments, ret
+end
+
 function split_iden (name)
    if name == '...' then return name end
    local pname,field = name:match('(.-)%.(.+)')
@@ -752,6 +838,9 @@ function build_arg_list (names,pmods)
    return  '('..table.concat(buffer)..')'
 end
 
+------ retrieving information about parameters -----
+-- The template leans on these guys heavily....
+
 function Item:param_modifiers (p)
    local mods = self.modifiers[self.parameter]
    if not mods then return '' end
@@ -771,9 +860,10 @@ function Item:default_of_param(p)
    return opt
 end
 
-function Item:type_of_ret(idx)
-   local rparam = self.modifiers['return'][idx]
-   return rparam and rparam.type or ''
+function Item:readonly(p)
+   local m = self:param_modifiers(p)
+   if not m then return nil end
+   return m.readonly
 end
 
 function Item:subparam(p)
@@ -794,6 +884,130 @@ function Item:display_name_of(p)
    end
 end
 
+-------- return values and types -------
+
+function Item:type_of_ret(idx)
+   local rparam = self.modifiers['return'][idx]
+   return rparam and rparam.type or ''
+end
+
+local function integer_keys(t)
+   if type(t) ~= 'table' then return 0 end
+   for k in pairs(t) do
+      local num = tonumber(k)
+      if num then return num end
+   end
+   return 0
+end
+
+function Item:return_type(r)
+   if not r.type then return '' end
+   return r.type, r.ctypes
+end
+
+local struct_return_type = '*'
+
+function Item:build_return_groups()
+   local modifiers = self.modifiers
+   local retmod = modifiers['return']
+   local groups = List()
+   local lastg, group
+   for i,ret in ipairs(self.ret) do
+      local mods = retmod[i]
+      local g = integer_keys(mods)
+      if g ~= lastg then
+         group = List()
+         group.g = g
+         groups:append(group)
+         lastg = g
+      end
+      --require 'pl.pretty'.dump(ret)
+      if not mods then
+         self:error(quote(self.name)..' had no return?')
+      end
+      group:append({text=ret, type = mods and (mods.type or '') or '',mods = mods})
+   end
+   -- order by groups to force error groups to the end
+   table.sort(groups,function(g1,g2) return g1.g < g2.g end)
+   self.retgroups = groups
+   --require 'pl.pretty'.dump(groups)
+   -- cool, now see if there are any treturns that have tfields to associate with
+   local fields = self.tags.field
+   if fields then
+      local fcomments = List()
+      for i,f in ipairs(fields) do
+         local name, comment = self:split_param(f)
+         fields[i] = name
+         fcomments[i] = comment
+      end
+      local fmods = modifiers.field
+      for group in groups:iter() do for r in group:iter() do
+         if r.mods and r.mods.type  then
+            local ctypes, T = List(), r.mods.type
+            for i,f in  ipairs(fields) do if fmods[i][T] then
+               ctypes:append {name=f,type=fmods[i].type,comment=fcomments[i]}
+            end end
+            r.ctypes = ctypes
+            --require 'pl.pretty'.dump(ctypes)
+         end
+      end end
+   end
+end
+
+local ecount = 0
+
+-- this alias macro implements @error.
+-- Alias macros need to return the same results as Item:check_tags...
+function doc.error_macro(tags,value,modifiers)
+   local merge_groups = doc.ldoc.merge_error_groups
+   local g = '2' -- our default group id
+   -- Were we given an explicit group modifier?
+   local key = integer_keys(modifiers)
+   if key > 0 then
+      g = tostring(key)
+   else
+      local l = tags:get 'return'
+      if l then -- there were returns already......
+         -- maximum group of _existing_ error return
+         local grp, lastr = 0
+         for r in l:iter() do if type(r) == 'table' then
+            local rg = r.modifiers._err
+            if rg then
+               lastr = r
+               grp = math.max(grp,rg)
+            end
+         end end
+         if grp > 0 then -- cool, create new group
+            if not merge_groups then
+               g = tostring(grp+1)
+            else
+               local mods, text, T = lastr.modifiers
+               local new = function(text)
+                  return mods._collected..' '..text,{type='string',[T]=true}
+               end
+               if not mods._collected then
+                  text = lastr[1]
+                  lastr[1] = merge_groups
+                  T = '@'..ecount
+                  mods.type = T
+                  mods._collected = 1
+                  ecount = ecount + 1
+                  tags:add('field',new(text))
+               else
+                  T = mods.type
+               end
+               mods._collected = mods._collected + 1
+               return 'field',new(value)
+            end
+         end
+      end
+   end
+   tags:add('return','',{[g]=true,type='nil'})
+   -- note that this 'return' is tagged with _err!
+   return 'return', value, {[g]=true,_err=tonumber(g),type='string'}
+end
+
+---------- bothering the user --------------------
 
 function Item:warning(msg)
    local file = self.file and self.file.filename
@@ -827,31 +1041,49 @@ end
 local err = io.stderr
 
 local function custom_see_references (s)
-    for pat, action in pairs(see_reference_handlers) do
-        if s:match(pat) then
-            local label, href = action(s:match(pat))
-            if not label then print('custom rule failed',s,pat,href) end
-            return {href = href, label = label}
-        end
-    end
+   for pat, action in pairs(see_reference_handlers) do
+      if s:match(pat) then
+         local label, href = action(s:match(pat))
+         if not label then print('custom rule failed',s,pat,href) end
+         return {href = href, label = label}
+      end
+   end
 end
 
 local function reference (s, mod_ref, item_ref)
    local name = item_ref and item_ref.name or ''
    -- this is deeply hacky; classes have 'Class ' prepended.
-   if item_ref and doc.class_tag(item_ref.type) then
-      name = 'Class_'..name
-   end
+--~    if item_ref and doc.class_tag(item_ref.type) then
+--~       name = 'Class_'..name
+--~    end
    return {mod = mod_ref, name = name, label=s}
 end
 
+function Module:lookup_class_item (packmod, s)
+   local klass = packmod --"Class_"..packmod
+   local qs = klass..':'..s
+   local klass_section = self.sections.by_name[klass]
+   if not klass_section then return nil end -- no such class
+   for item in self.items:iter() do
+      --print('item',qs,item.name)
+      if s == item.name or qs == item.name then
+         return reference(s,self,item)
+      end
+   end
+   return nil
+end
+
 function Module:process_see_reference (s,modules,istype)
+   if s == nil then return nil end
    local mod_ref,fun_ref,name,packmod
    local ref = custom_see_references(s)
    if ref then return ref end
    if not s:match '^[%w_%.%:%-]+$' or not s:match '[%w_]$' then
       return nil, "malformed see reference: '"..s..'"'
    end
+
+   -- `istype` means that we are looking up strictly in a _type_ context, so then only
+   -- allow `classmod` module references.
    local function ismod(item)
       if item == nil then return false end
       if not istype then return true
@@ -859,6 +1091,16 @@ function Module:process_see_reference (s,modules,istype)
          return item.type == 'classmod'
       end
    end
+
+   -- it is _entirely_ possible that someone does not want auto references for standard Lua libraries!
+   local lua_manual_ref
+   local ldoc = tools.item_ldoc(self)
+   if ldoc and ldoc.no_lua_ref then
+      lua_manual_ref = function(s) return false end
+   else
+      lua_manual_ref = global.lua_manual_ref
+   end
+
    -- is this a fully qualified module name?
    local mod_ref = modules.by_name[s]
    if ismod(mod_ref) then return reference(s, mod_ref,nil) end
@@ -875,12 +1117,20 @@ function Module:process_see_reference (s,modules,istype)
       if not mod_ref then
          mod_ref = self:hunt_for_reference(packmod, modules)
          if not mod_ref then
-            local ref = global.lua_manual_ref(s)
+            local ref = self:lookup_class_item(packmod,s)
+            if ref then return ref end
+            local mod, klass = split_dotted_name(packmod)
+            mod_ref = modules.by_name[mod]
+            if mod_ref then
+               ref = mod_ref:lookup_class_item(klass,name)
+               if ref then return ref end
+            end
+            ref = lua_manual_ref(s)
             if ref then return ref end
             return nil,"module not found: "..packmod
          end
       end
-      fun_ref = mod_ref.items.by_name[name]
+      fun_ref = mod_ref:get_fun_ref(name)
       if fun_ref then
          return reference(s,mod_ref,fun_ref)
       else
@@ -894,16 +1144,32 @@ function Module:process_see_reference (s,modules,istype)
    else -- plain jane name; module in this package, function in this module
       mod_ref = modules.by_name[self.package..'.'..s]
       if ismod(mod_ref) then return reference(s, mod_ref,nil) end
-      fun_ref = self.items.by_name[s]
-      if fun_ref then return reference(s, self,fun_ref)
+      fun_ref = self:get_fun_ref(s)
+      if fun_ref then return reference(s,self,fun_ref)
       else
-         local ref = global.lua_manual_ref (s)
+         local ref = lua_manual_ref (s)
          if ref then return ref end
          return nil, "function not found: "..s.." in this module"
       end
    end
 end
 
+function Module:get_fun_ref(s)
+   local fun_ref = self.items.by_name[s]
+   -- did not get an exact match, so try to match by the unqualified fun name
+   if not fun_ref then
+      local patt = '[.:]'..s..'$'
+      for qname,ref in pairs(self.items.by_name) do
+         if qname:match(patt) then
+            fun_ref = ref
+            break
+         end
+      end
+   end
+   return fun_ref
+end
+
+
 -- resolving @see references. A word may be either a function in this module,
 -- or a module in this package. A MOD.NAME reference is within this package.
 -- Otherwise, the full qualified name must be used.
@@ -935,14 +1201,6 @@ function Module:resolve_references(modules)
    end
 end
 
--- suppress the display of local functions and annotations.
--- This is just a placeholder hack until we have a more general scheme
--- for indicating 'private' content of a module.
-function Module:mask_locals ()
-   self.kinds['Local Functions'] = nil
-   self.kinds['Annotations'] = nil
-end
-
 function Item:dump_tags (taglist)
    for tag, value in pairs(self.tags) do
       if not taglist or taglist[tag] then
